// @ts-check

const path = require("path");
const fs = require("fs");
const grabHostNames = require("../package-shared/utils/grab-host-names");
const apiCreateUser = require("../package-shared/functions/api/users/api-create-user");

/**
 * Add User to Database
 * ============================================
 * @async
 *
 * @param {object} param - Single object passed
 * @param {string} [param.key] - FULL ACCESS API Key
 * @param {string} [param.database] - Database Name
 * @param {import("../package-shared/types").UserDataPayload} param.payload - User Data Payload
 * @param {string} [param.encryptionKey]
 * @param {string} [param.encryptionSalt]
 * @param {string | number} [param.user_id]
 * @param {string | number} [param.apiUserId]
 * @param {boolean} [param.useLocal]
 *
 * @returns { Promise<import("../package-shared/types").AddUserFunctionReturn> }
 */
async function addUser({
    key,
    payload,
    database,
    encryptionKey,
    user_id,
    useLocal,
    apiUserId,
}) {
    /**
     * Check for local DB settings
     *
     * @description Look for local db settings in `.env` file and by pass the http request if available
     */
    const {
        DSQL_DB_HOST,
        DSQL_DB_USERNAME,
        DSQL_DB_PASSWORD,
        DSQL_DB_NAME,
        DSQL_API_USER_ID,
    } = process.env;

    const grabedHostNames = grabHostNames();
    const { host, port, scheme } = grabedHostNames;

    if (
        DSQL_DB_HOST?.match(/./) &&
        DSQL_DB_USERNAME?.match(/./) &&
        DSQL_DB_PASSWORD?.match(/./) &&
        DSQL_DB_NAME?.match(/./) &&
        useLocal
    ) {
        /** @type {import("../package-shared/types").DSQL_DatabaseSchemaType | undefined} */
        let dbSchema;

        try {
            const localDbSchemaPath = path.resolve(
                process.cwd(),
                "dsql.schema.json"
            );
            dbSchema = JSON.parse(fs.readFileSync(localDbSchemaPath, "utf8"));
        } catch (error) {}

        return await apiCreateUser({
            database: DSQL_DB_NAME,
            encryptionKey,
            payload,
            userId: apiUserId,
            useLocal,
        });
    }

    /**
     * Make https request
     *
     * @description make a request to datasquirel.com
     */
    const httpResponse = await new Promise((resolve, reject) => {
        const reqPayload = JSON.stringify({
            payload,
            database,
            encryptionKey,
        });

        const httpsRequest = scheme.request(
            {
                method: "POST",
                headers: {
                    "Content-Type": "application/json",
                    "Content-Length": Buffer.from(reqPayload).length,
                    Authorization:
                        key ||
                        process.env.DSQL_FULL_ACCESS_API_KEY ||
                        process.env.DSQL_API_KEY,
                },
                port,
                hostname: host,
                path: `/api/user/${
                    user_id || grabedHostNames.user_id
                }/add-user`,
            },

            /**
             * Callback Function
             *
             * @description https request callback
             */
            (response) => {
                var str = "";

                response.on("data", function (chunk) {
                    str += chunk;
                });

                response.on("end", function () {
                    resolve(JSON.parse(str));
                });

                response.on("error", (err) => {
                    reject(err);
                });
            }
        );
        httpsRequest.write(reqPayload);
        httpsRequest.end();
    });

    return httpResponse;
}

module.exports = addUser;
